"""
Clase que se encarga de pintar el fondo, sobre la que se pintara todo lo demas, y la que gestionara el scroll
"""

import pygame
import random
import copy

class Background(pygame.sprite.Sprite):
    def __init__(self, map, screen):
        pygame.sprite.Sprite.__init__(self)
        self.size = (map.get_properties()[0], map.get_properties()[1])
        self.screen = screen
        self.scroll_speed = 10
        self.scroll_width = 20
        self.image = pygame.Surface(self.size)
        self.relleno_estrella()
        self.orig = copy.copy(self.image)
        self.rect = self.image.get_rect()
        self.rect.center = (self.screen.get_width()/2, self.screen.get_height()/2)

    def get_image(self):
        return self.image
    def get_rect(self):
        return self.rect
    def get_scroll(self):
        return self.rect.left, self.rect.top

    def update(self):
        self.image = copy.copy(self.orig)
        x, y = pygame.mouse.get_pos()

        if x <= self.scroll_width and self.rect.left < 0:
            self.rect.left += self.scroll_speed
        elif x >= self.screen.get_width() - self.scroll_width and self.rect.right > self.screen.get_width():
            self.rect.right -= self.scroll_speed
        if y <= self.scroll_width and self.rect.top < 0:
            self.rect.top += self.scroll_speed
        elif y >= self.screen.get_height() - self.scroll_width and self.rect.bottom > self.screen.get_height():
            self.rect.bottom -= self.scroll_speed


    def set_pos(self, (x, y)):
        """
        centra en la pantalla x, y
        """

        mx = self.screen.get_width()
        my = self.screen.get_height()
        self.rect.left = -(x - mx/2)
        self.rect.top = -(y - my/2)
        if self.rect.left > 0 and self.rect.right > mx:
            self.rect.left = 0
        elif self.rect.right < mx and self.rect.left < 0:
            self.rect.right = mx
        if self.rect.top > 0 and self.rect.bottom > my:
            self.rect.top = 0
        elif self.rect.bottom < my and self.rect.top < 0:
            self.rect.bottom = my


    def relleno_estrella(self):
        w = self.size[0]
        h = self.size[1]
        for i in range(w / 2):
            random.seed()
            centerx = random.randint(0, w)
            centery = random.randint(0, h)
            rect = pygame.Rect(centerx, centery, 2, 2)
            pygame.draw.rect(self.image, (255,255,255), rect)


