import Player
import Unit
import math
import utils
import random

class IA(Player.Player):
    """
    Interfaz para programar inteligencias. Tan solo hay que
    redefinir la funcion update
    """

    sleep_init = 40
    DIFICULTAD = 3

    def __init__(self, nombre, color):
        self.nombre = nombre
        self.color = color
        self.ataque = 50
        self.planetas = []
        self.seleccionado = None #planeta seleccionado
        self.amigos = []

    def atacar_planeta(self, destino, allsprites):
        units = Unit.create_units(None, self.get_units(), self, destino, allsprites)
        self.seleccionado.salida_unidades(self.get_units())
        return units

    def get_units(self):
        return int(self.seleccionado.get_size() * (self.ataque / 100.0))

    def update(self, allsprites):
        """
        dummy update
        """
        pass
        return None
        
    def buscar_mios(self, porcentaje):
        """
        busca en la lista de planetas y devuelve los que son del player con mas de un "porcentaje" de unidades
        y los que no son del player, como otra lista.
        """
        mios = []
        no_mios = []
        for planeta in self.planetas:
            if planeta.get_habitante() == None:
                no_mios.append(planeta)
            elif not planeta.get_habitante().nombre == self.nombre:
                no_mios.append(planeta)
            elif planeta.get_porcentaje() > porcentaje:
                mios.append(planeta)
        return mios, no_mios

    def buscar_friends(self):
        '''
        Busca los planetas que pertenecen a algun aliado
        '''
        friends = []
        amigos = self.get_friends()
        for p in self.planetas:
            if p.get_habitante() in amigos:
                friends.append(p)
        return friends

    def buscar_mas_debil(self):
        """
        busca en la lista de planetas el planeta no ocupado por mi, mas debil
        """
        mas_debil = self.planetas[0]
        amigos = self.get_friends()
        for planeta in self.planetas:
            if not planeta.get_habitante() in amigos + [self]:
                if planeta.get_size() < mas_debil.get_size():
                    mas_debil = planeta
        return mas_debil

    def distancia(self, origen, destino):
        """
        calcula la distancia entre dos planetas
        """
        x, y = origen.get_pos()
        z, t = destino.get_pos()
        zx = z - x
        ty = t - y
        return math.sqrt(zx * zx + ty * ty)

class IA1(IA):
    """
    Inteligencia Artificial 1: 
    - Ataca primero a los planetas mas cercanos,
    - manteniendo siempre un 30% en sus planetas conquistados
    - En principio ataca siempre con un 30%
    """
    def __init__(self, nombre, color):
        IA.__init__(self, nombre, color)
        self.set_ataque(30)
        self.sleep = IA.sleep_init * IA.DIFICULTAD

    def update(self, allsprites):
        """
        - buscamos planeta, que no sea mio, mas cercano a uno nuestro con mas del 30%
            - si -> atacamos
            - no -> pasamos
        """
        if self.sleep > 0:
            self.sleep -= 1
            return None

        mios, no_mios = self.buscar_mios(30) #se le pasa como limite el 30% de unidades
        if len(mios) > 0 and len(no_mios) > 0:
            origen = mios[0]
            objetivo = no_mios[0]
            distancia = self.distancia(origen, objetivo)
            amigos = self.buscar_friends()
            for mio in mios:
                for nomio in no_mios:
                    if nomio in amigos:
                        continue
                    dist_tmp = self.distancia(mio, nomio)
                    if distancia > dist_tmp:
                        origen = mio
                        objetivo = nomio
                        distancia = dist_tmp
            self.seleccionado = origen
            self.sleep = IA.sleep_init * IA.DIFICULTAD
            return self.atacar_planeta(objetivo, allsprites)

        else: return None

class IA2(IA):
    """
    Inteligencia Artificial agresiva:
    - Ataca a los planetas mas cercanos
    - Ataca con un 10% de unidades
    - mantiene siempre un 20% en sus planetas conquistados
    """
    def __init__(self, nombre, color):
        IA.__init__(self, nombre, color)
        self.set_ataque(70)

        self.sleep = IA.sleep_init * IA.DIFICULTAD

    def update(self, allsprites):
        if self.sleep > 0:
            self.sleep -= 1
            return None

        mios, no_mios = self.buscar_mios(10)
        if len(mios) > 0 and len(no_mios) > 0:
            origen = mios[0]
            objetivo = no_mios[0]
            distancia = self.distancia(origen, objetivo)
            amigos = self.buscar_friends()
            for mio in mios:
                for nomio in no_mios:
                    if nomio in amigos:
                        continue
                    dist_tmp = self.distancia(mio, nomio)
                    if distancia > dist_tmp:
                        origen = mio
                        objetivo = nomio
                        distancia = dist_tmp
            self.seleccionado = origen
            self.sleep = IA.sleep_init * IA.DIFICULTAD
            return self.atacar_planeta(objetivo, allsprites)

        else: return None

class IA3(IA):
    def __init__(self, nombre, color):
        IA.__init__(self, nombre, color)
        self.set_ataque(30)
        self.sleep = IA.sleep_init * IA.DIFICULTAD

    def update(self, allsprites):
        """
        Super Agresivo, y cabron
        ataca al planeta mas debil
        con la menor division de la distancia por el numero de unidades d / s
        y con 30% de unidades
        """

        if self.sleep > 0:
            self.sleep -= 1
            return None

        debil = self.buscar_mas_debil()
        mios, no_mios = self.buscar_mios(10)
        if len(mios) > 0:
            origen = mios[0]
            distancia = self.distancia(origen, debil)
            unidades = origen.get_size() + 1
            for mio in mios:
                dist_tmp = self.distancia(mio, debil)
                unid_tmp = mio.get_size() + 1
                if dist_tmp / unid_tmp < distancia / unidades:
                    origen = mio
                    distancia = dist_tmp
                    unidades = unid_tmp
            if distancia > 0:
                self.seleccionado = origen
                self.sleep = IA.sleep_init * IA.DIFICULTAD
                return self.atacar_planeta(debil, allsprites)
        return None

class IA4(IA):
    def __init__(self, nombre, color):
        IA.__init__(self, nombre, color)
        self.set_ataque(30)
        self.limite = 0

        self.sleep = IA.sleep_init * IA.DIFICULTAD

    def update(self, allsprites):
        """
        ataca al planeta mas debil
        con la menor division de la distancia por el numero de unidades d / s
        y con 30% de unidades
        """
        if self.sleep > 0:
            self.sleep -= 1
            return None

        debil = self.buscar_mas_debil()
        mios, no_mios = self.buscar_mios(self.limite)
        if len(mios) > 0:
            origen = mios[0]
            distancia = self.distancia(origen, debil)
            unidades = origen.get_size() + 1
            for mio in mios:
                dist_tmp = self.distancia(mio, debil)
                unid_tmp = mio.get_size() + 1
                if dist_tmp / unid_tmp < distancia / unidades:
                    origen = mio
                    distancia = dist_tmp
                    unidades = unid_tmp
            if unidades < 20:
                self.limite += 1
            elif unidades > 50:
                self.limite -= 1
            if distancia > 0:
                self.seleccionado = origen
                self.sleep = IA.sleep_init * IA.DIFICULTAD
                return self.atacar_planeta(debil, allsprites)
        return None


class IA5(IA):
    def __init__(self, nombre, color):
        IA.__init__(self, nombre, color)
        self.set_ataque(30)
        self.limite = 0
        self.unitsprites = []

        self.units_ant = 0

        self.sleep = IA.sleep_init * IA.DIFICULTAD
        self.inteligencias = [IA1, IA2, IA3, IA4]
        self.intel = random.choice(self.inteligencias)(nombre, color)

    def __setattr__(self, nombre, valor):
        self.__dict__[nombre] = valor

        if nombre == 'planetas': 
            if valor != []:
                self.intel.planetas = valor

    def update(self, allsprites):
        """
        Cambia de estrategia si detecta que esta perdiendo
        """
        if self.intel.sleep > 0:
            self.intel.sleep -= 1
            return None

        nUnits = 0
        for p in self.planetas:
            if p.habitante == self:
                nUnits += p.size
        nUnits_vuelo = 0
        for u in self.unitsprites:
            if u.player == self:
                nUnits_vuelo += u.size
        nUnits_total = nUnits_vuelo + nUnits

        if nUnits_total < self.units_ant:
            self.intel = random.choice(self.inteligencias)(self.nombre, self.color)
            self.intel.planetas = self.planetas

            self.units_ant = nUnits_total
        
        print self.intel
        to_ret = self.intel.update(allsprites)
        return to_ret
