"""
Mini mapa que representa el mapa en general, para poder moverte por la pantalla
"""

import pygame

class Minimap(pygame.sprite.Sprite):
    def __init__(self, map, units, screen, background):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.map = map
        self.units = units
        self.screen = screen
        self.background = background

        bsize = background.size
        ssize = screen.get_width(), screen.get_height()

        self.show = False
        if bsize[0] > ssize[0] or bsize[1] > ssize[1]:
            self.show = True

        self.size = (150, 100)
        self.calc_mini_rect()
        self.image = pygame.Surface((0,0))
        self.rect = self.image.get_rect()
        if self.show:
            self.update_planets()
        self.drag = False
        


    def update(self):
        if self.show:
            pos = self.inside(pygame.mouse.get_pos())
            if self.drag and pos:
                self.background.set_pos(self.calc_real_pos(pos))
            self.update_planets()
            self.draw_units()

    def inside(self, pos):
        """
        comprueba si una posicion esta dentro de los limites del minimap, y devuelve la posicion relativa
        """

        if self.show:
            x, y = pos
            if self.rect.left + 2 <= x <= self.rect.right - 2 and \
            self.rect.top + 2 <= y <= self.rect.bottom - 2:
                return x - self.rect.left - 1, y - self.rect.top - 1
            else:
                return None
        else:
            return None


    def update_planets(self):
        """
        redibuja todos los planetas
        """
        self.draw_rectangle()
        self.planets = self.map.get_planets()
        for planet in self.planets:
            pos = self.calc_mini_pos(planet.get_pos())
            size = planet.get_radio()
            color = planet.get_color()
            if planet.get_selected():
                color = (255,255,0)
            pygame.draw.circle(self.image, color, pos, size/5)

        self.draw_mini_rectangle()

        self.rect = self.image.get_rect()
 
        self.rect.right = self.screen.get_width()
        self.rect.bottom = self.screen.get_height()


    def draw_rectangle(self):
        """
        dibuja un cuadro pequenio en la esquina inferior derecha, con borde rojo
        """
        self.image = pygame.Surface((self.size[0] + 4, self.size[1] + 4))
        self.image.fill((0,0,0))
        self.image.set_alpha(200)
        rectangle = [(0, 0), (self.size[0], 0), (self.size[0], self.size[1]), (0, self.size[1])]
        self.rect = pygame.draw.polygon(self.image, (255,0,0), rectangle, 2)

        self.rect = self.image.get_rect()

        self.rect.right = self.screen.get_width()
        self.rect.bottom = self.screen.get_height()

    def draw_mini_rectangle(self):
        """
        dibuja el recuadro que representa al screen
        """
        scrollx, scrolly = self.background.get_scroll()
        mw = self.map.get_properties()[0]
        mh = self.map.get_properties()[1]

        posx = scrollx * (self.size[0] - 1) / mw
        posy = scrolly * (self.size[1] - 1) / mh
        posx = -posx
        posy = -posy
        rectangle = [(posx, posy), (posx + self.mini_width, posy), \
        (posx + self.mini_width, posy + self.mini_height), \
        (posx, posy + self.mini_height)]
        
        pygame.draw.polygon(self.image, (255,255,0), rectangle, 1)

    def calc_mini_rect(self):
        w = self.screen.get_width()
        h = self.screen.get_height()
        mw = self.map.get_properties()[0]
        mh = self.map.get_properties()[1]
        self.mini_width = w * (self.size[0] - 1) / mw
        self.mini_height = h * (self.size[1] - 1) / mh

    def draw_unit(self, unit):
        munit = pygame.Surface((2,2))
        munit.fill(unit.get_color())
        pos = self.calc_mini_pos(unit.get_pos())
        self.image.blit(munit, pos)

    def draw_units(self):
        for i in self.units:
            self.draw_unit(i)

    def calc_mini_pos(self, (x, y)):
        x = x * (self.size[0] - 1) / self.map.get_properties()[0]
        y = y * (self.size[1] - 1) / self.map.get_properties()[1]
        return x, y

    def calc_real_pos(self, (x, y)):
        x = self.map.get_properties()[0] * x / (self.size[0] - 1)
        y = self.map.get_properties()[1] * y / (self.size[1] - 1)
        return x, y

    def set_drag(self):
        self.drag = True
    def unset_drag(self):
        self.drag = False
