"""
Clase que representa un planeta. Un punto de conquista, y creacion
de unidades
"""

import pygame
import utils
import Unit
import time
from copy import *
import os

class Planet(pygame.sprite.Sprite):
    """
    Clase que representa un planeta.
    """

    image = None

    def __init__(self, center, size=100, habitante=None, name=''):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.size = size
        self.orig_size = size
        self.escala = self.size / 100.0
        #tamanio del planeta, determina la resistencia, y el numero maximo de unidades

        path = os.path.join(utils.data_dir, 'planet.png')
        path_sombra = os.path.join(utils.data_dir, 'sombra.png')
        if Planet.image is None:
            self.orig = pygame.image.load(path)
            self.sombra = pygame.image.load(path_sombra)
            self.orig = self.orig.convert_alpha()
            self.sombra = self.sombra.convert_alpha()
            self.orig.blit(self.sombra, (0,0))
            Planet.image = self.orig
        else:
            self.orig = Planet.image

        self.orig = pygame.transform.scale(self.orig, (int(self.orig.get_width()*self.escala), int(self.orig.get_height()*self.escala)))
        self.rect = self.orig.get_rect()
        self.center = center
        self.rect.center = center
        self.image1 = copy(self.orig)
        self.image = copy(self.orig)
        #la imagen del planeta
        
        path = os.path.join(utils.data_dir, 'font.ttf')
        self.font = pygame.font.Font(path, utils.font_size)
        self.text = self.font.render(str(self.size), 1, (250,250,250,250))
        self.textpos = self.rect.centerx
        #en el centro del planeta se va a escribir el numero de unidades
        self.habitante = habitante
        self.name = name
        self.text_name = self.font.render(name, 1, (250,250,250,250))
        
        self.reload()

        self.inicial = time.time()
        self.sec = 0
        self.selected = False

    def reload(self):
        marron = (175, 71, 0)
        if self.habitante is None:
            self.colorear(marron)
        else:
            self.colorear(self.habitante.get_color())

    def set_bg(self, bg):
        self.bg = bg

    def update(self):
        """
        Cada segundo que pasa, si esta ocupado el planeta, el numero
        de unidades se incrementa en 1
        """

        if self.habitante is not None:
            self.incrementar()
 
        w = self.image1.get_width()
        h = self.image1.get_height()
        h2 = h + self.text_name.get_height() + 10
        w2 = self.text_name.get_width()
        if w2 < w:
            w2 = w
        self.image = pygame.Surface((w2, h2))
        self.image.set_colorkey((0,0,0))
        self.image.blit(self.image1, ((w2-w)/2,0))
        if self.habitante is None or self.habitante.nombre == 'humanos':
            self.text = self.font.render(str(self.size), 1, (250,250,250,250))
            x = w2/2 - self.text.get_width()/2
            y = h/2 - self.text.get_height()/2
            self.image.blit(self.text,(x, y))

        if hasattr(self, 'bg') and self.encima():
            x = w2/2 - self.text_name.get_width()/2
            y = h + 10
            self.image.blit(self.text_name,(x, y))

        if self.size > self.orig_size*2:
            self.size = self.orig_size*2
        
    def colorear(self, color):
        """
        Pone una capa semitransparente sobre la imagen del planeta, le da color
        """
        self.color = color
        self.image1 = copy(self.orig)
        surface = pygame.Surface((self.image1.get_width(), self.image1.get_height()))
        surface.fill((0,0,0))
        surface.set_colorkey((0,0,0))
        pygame.draw.circle(surface, color, (surface.get_width()/2,\
            surface.get_height()/2), (surface.get_height()+1)/2)
        surface.set_alpha(160)
        self.image1.blit(surface, (0, 0))
        
        #si esta habitado, se le pone un circulo alrededor
        if self.habitante is not None:
            surface = pygame.Surface((self.image1.get_width()+10, self.image1.get_height()+10))
            surface.fill((0,0,0))
            surface.set_colorkey((0,0,0))
            pygame.draw.circle(surface, color, (surface.get_width()/2, surface.get_height()/2), surface.get_width()/2, 1)
            surface.blit(self.image1, (5,5))
            self.image1 = surface

        self.image = copy(self.image1)
        self.rect = self.image.get_rect()
        self.rect.center = self.center

    def incrementar(self):
        """
        Incrementa cada segundo el numero de unidades, y lo escribe en el planeta
        """
        if self.size < self.orig_size * 2:
            act = self.sec
            self.sec = int(time.time() - self.inicial)
            if not act == self.sec:
                self.size+=1 + int(self.orig_size/200)

    def get_pos(self):
        return self.rect.center

    def get_size(self):
        return self.size

    def get_radio(self):
        return self.image.get_width()/2

    def select(self):
        if self.habitante is not None:
            surface = pygame.Surface((self.image1.get_width()+10, self.image1.get_height()+10))
            surface.set_colorkey(surface.get_at((0,0)))
            pygame.draw.circle(surface, (255,255,0),
                    (surface.get_width()/2, surface.get_height()/2),\
                    surface.get_width()/2, 1)
            surface.blit(self.image1, (5,5))
            self.image1 = surface
            self.selected = True

        self.image = copy(self.image1)
        self.rect = self.image.get_rect()
        self.rect.center = self.center

    def unselect(self):
        if self.habitante is not None:
            try:
                # player1
                if self in self.habitante.get_selected():
                    self.habitante.unset_selected(self)
            except:
                # ia
                self.habitante.unset_selected(None)

            self.colorear(self.habitante.get_color())
            self.selected = False

    def salida_unidades(self, unidad):
        self.size -= unidad

    def llegada_unidades(self, unidad):
        if self.habitante is not unidad.get_player():
            self.size -= unidad.get_size()
            if self.size < 0:
                self.size = -self.size
                self.unselect()
                self.habitante = unidad.get_player()
                self.colorear(self.habitante.get_color())
                return True
        else:
            self.size += unidad.get_size()
        unidad.explotar()
        unidad.kill()

        return False

    def get_habitante(self):
        return self.habitante

    def get_color(self):
        return self.color

    def get_selected(self):
        return self.selected

    def get_porcentaje(self):
        return (self.size * 100 / float(self.orig_size))

    def get_name(self):
        return self.name

    def encima(self):
        x, y = pygame.mouse.get_pos()
        scrollx, scrolly = self.bg.get_scroll()
        
        x -= scrollx
        y -= scrolly

        xp, yp = self.get_pos()
        if xp - self.get_radio() < x < xp + self.get_radio() and \
        yp - self.get_radio() < y < yp + self.get_radio():
            return True
        else:
            return False
