# -*- coding: utf-8 -*-
"""
Lee el fichero estandar de mapa
"""

from xml.dom import minidom
import Planet
import Map
import target

class ReadXmlMap():
    def __init__(self, file, players):
        self.file = file
        self.map_width = 0
        self.map_height = 0
        self.map_title = 0
        self.planetas = []
        self.players = players
        self.targets = []

    def read(self):
        """
        Lee el documento xml, y devuelve un Map
        """
        dom = minidom.parse(self.file)
        #leemos la etiqueta mapa, que define el tamanio
        for i in dom.getElementsByTagName("map"):
            for j in i.attributes.keys():
                if j == "width":
                    self.map_width=int(i.attributes[j].value)
                if j == "height":
                    self.map_height=int(i.attributes[j].value)
        #leemos la etiqueta de titulo
        for i in dom.getElementsByTagName("title"):
            self.map_title = i.childNodes[0].nodeValue

        for obj in dom.getElementsByTagName("target"):
            t = target.Target()
            for attr in obj.attributes.keys():
                if attr == 'type':
                    t.type = obj.attributes[attr].value
                elif attr == 'dst':
                    t.dst = obj.attributes[attr].value
                elif attr == 'level':
                    t.level = int(obj.attributes[attr].value)

            t.text = obj.childNodes[0].nodeValue
            self.targets.append(t)

        #leemos cada linea, que representa un planeta
        for planet in dom.getElementsByTagName("planet"):
            for attr in planet.attributes.keys():
                if attr == "x":
                    x = int(planet.attributes[attr].value)
                if attr == "y":
                    y = int(planet.attributes[attr].value)
                if attr == "size":
                    size = int(planet.attributes[attr].value)
                if attr == "habitado":
                    habitant = planet.attributes[attr].value
                if attr == "name":
                    name = planet.attributes[attr].value

            self.planetas.append(Planet.Planet((x,y), size,\
                self.buscar_player(habitant), name=name))

        for aliado in dom.getElementsByTagName("allied"):
            alianza = []
            for attr in aliado.attributes.keys():
                if attr == "player1" or attr == "player2":
                    alianza.append(aliado.attributes[attr].value)

            player1 = self.buscar_player(alianza[0])
            player2 = self.buscar_player(alianza[1])
            player1.add_friend(player2)
            player2.add_friend(player1)

        dom.unlink()
        return Map.Map((self.map_width, self.map_height), self.map_title, self.planetas)

    def get_planetas(self):
        return self.planetas
    def get_map_properties(self):
        return self.map_width, self.map_height, self.map_title
    def buscar_player(self, name):
        for i in self.players:
            if i.get_name() == name:
                return i
        return None
    def get_targets(self):
        return self.targets

    def get_load_random_map(self):
        import random

        w = 1000
        h = 1000
        def cercano(x, list):
            print x, list
            for i in list:
                if i + 40 < x or x < i - 40:
                    continue
                else:
                    return True
            return False

        planets = random.randint(8,14)
        xs = []
        ys = []

        self.map_width=w
        self.map_height=w
        self.map_title = "Random Map"

        t = target.Target()
        t.type = "defeat all"
        t.text = "Acaba con todos los enemigos"
        self.targets.append(t)

        for planet in range(planets):
            x = random.randint(100, w-100)
            while cercano(x, xs):
                x = random.randint(100, w-100)
            xs.append(x)
            y = random.randint(100, h-100)
            while cercano(y, ys):
                y = random.randint(100, h-100)
            ys.append(y)

            size = random.randint(80, 140)
            habitant = "None"
            name = "planeta"+str(planet)

            self.planetas.append(Planet.Planet((x,y), size, None, name=name))
        print xs

        cogidos = []
        for p in self.players:
            i = random.choice(self.planetas)
            while i in cogidos:
                i = random.choice(self.planetas)

            cogidos.append(i)

            i.habitante = p
            i.reload()

        for p in self.planetas:
            if p.habitante is None:
                p.size = random.randint(5,60)

        return Map.Map((self.map_width, self.map_height), self.map_title, self.planetas)


