# -*- coding: utf-8 -*-
'''
Muestra informacion al usuario
Objetivos, has ganado, has perdido, ese tipo de cosas
'''

import utils
import pygame
import os
class Info(pygame.sprite.Sprite):
    def __init__(self, screen, title, text, big=False):
        pygame.sprite.Sprite.__init__(self)
        self.screen = screen
        self.w = screen.get_width() - screen.get_width()*20/100
        self.h = screen.get_height() - screen.get_height()*20/100
        self.image = pygame.Surface((self.w, 10))
        self.rect = None
        self.title = title
        self.big = big
        self.Body = text
        self.die = False
        self.x = self.screen.get_width()/2 - self.w/2
        self.y = 50
        self.font_src = os.path.join(utils.data_dir, 'font.ttf')

    def update(self):
        w = self.w
        h = self.image.get_height()
        if h < self.h and not self.die:
            h += 20
            self.draw_rectangle(w, h)
        elif h > 10 and self.die:
            h -= 20
            self.draw_rectangle(w, h)
        elif self.die:
            pygame.sprite.Sprite.kill(self)
            return True
        return False

    def scroll_down(self):
        self.y -= 10
        w = self.w
        h = self.image.get_height()
        self.draw_rectangle(w, h)

    def scroll_up(self):
        self.y += 10
        w = self.w
        h = self.image.get_height()
        self.draw_rectangle(w, h)

    def scroll(self, event):
        if event.type == pygame.KEYDOWN and event.key == pygame.K_UP:
            self.scroll_up()
        elif event.type == pygame.KEYDOWN and event.key == pygame.K_DOWN:
            self.scroll_down()
        elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 4:
            self.scroll_up()
        elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 5:
            self.scroll_down()


    def kill(self):
        self.die = True

    def draw_rectangle(self, w, h):
        self.image = pygame.Surface((w, h))
        self.image.fill((0,0,0))
        self.image.set_alpha(180)
        rectangle = [(0, 0), (w-2, 0), (w-2, h-2), (0, h-2)]
        self.rect = pygame.draw.polygon(self.image, (0, 255,0), rectangle, 2)

        self.draw_text(self.Body, (self.x, self.y))
        self.draw_title(self.title)
        
        self.rect = self.image.get_rect()

        self.rect.centerx = self.screen.get_width()/2
        self.rect.centery = self.screen.get_height()/2

    def draw_title(self, text):
        font = pygame.font.Font(self.font_src, utils.font_size_big)
        color = (0,0,0)
        offset = 10
        txt = font.render(text, 1, color)
        x = 0
        y = 0
        h = txt.get_height()
        w = self.image.get_width() 
        image = pygame.Surface((w, h+offset))
        image.fill((255,0,0))
        image.set_alpha(255)
        rectangle = [(0, 0), (w-2, 0), (w-2, h+offset), (0, h+offset)]
        self.image.blit(image, (x,y))
        rect = pygame.draw.polygon(self.image, (255, 255, 0), rectangle, 2)
        self.image.blit(txt, (w/2-txt.get_width()/2, y+offset))

        
    def draw_text(self, text, (x,y)):
        text_size = utils.font_size
        if self.big:
            text_size = utils.font_size_big
        font = pygame.font.Font(self.font_src, text_size)
        color = (0,255,0)
        text = text.split('\n')
        max_w = self.image.get_width() - self.image.get_width()*0.2
        j = 0
        for i,t in enumerate(text):
            txt = font.render(t, 1, color)
            if txt.get_width() > max_w:
                txt_list = t.split()
                while len(txt_list) > 0:
                    j+=1
                    txt = font.render(txt_list[0], 1, color)
                    k = 1
                    while txt.get_width() < max_w and len(txt_list) > 1:
                        txts = ' '.join(txt_list[0:k])
                        txt = font.render(txts, 1, color)
                        k += 1

                    txts = ' '.join(txt_list[0:k])
                    txt_list = txt_list[k:]
                    txt = font.render(txts, 1, color)
                    self.image.blit(txt, (x+5, y + (i+j)*txt.get_height()+5))

                
            else:
                self.image.blit(txt, (x+5, y + (i+j)*txt.get_height()+5))


    def show(self, screen):
        import random
        # Fondo de estrellas
        w = self.screen.get_width()
        h = self.screen.get_height()
        surface = pygame.Surface((w,h))
        surface.fill((0,0,0))
        for i in range(w / 2):
            random.seed()
            centerx = random.randint(0, w)
            centery = random.randint(0, h)
            rect = pygame.Rect(centerx, centery, 2, 2)
            pygame.draw.rect(surface, (255,255,255), rect)

        bg_path = os.path.join(utils.data_dir, 'background.png')
        image = pygame.image.load(bg_path)
        image = image.convert_alpha()
        x = w/2 - 100
        y = -10
        surface.blit(image, (x, y))

        nfo = self
        while True:
            for event in pygame.event.get():
                nfo.scroll(event)
                if event.type == pygame.QUIT:
                    return 0
                elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    return 0

            screen.blit(surface, (0,0))
            nfo.update()
            screen.blit(nfo.image, nfo.rect)
            pygame.display.flip()
