import pygame
import copy

class SelRec:
    '''
    Un rectangulo para seleccionar varios planetas
    '''
    def __init__(self, player, planetas):
        self.player = player
        self.planetas = planetas
        self.rect = None


    def get(self):
        return self.rect is not None

    def set(self, (l,t), (w,h)):
        self.rect = pygame.Rect((l,t),(w,h))

    def draw(self, (posx, posy), surf):
        x, y = self.rect.left, self.rect.top
        self.rect.width = posx - x
        self.rect.height = posy - y

        cuadro = self.cuadro_verdadero()
        image = pygame.Surface((cuadro.width, cuadro.height))
        image.fill(self.player.get_color())
        image.set_alpha(30)
        image = image.convert()

        surf.blit(image, cuadro)
        pygame.draw.rect(surf, (255,255,0), self.rect, 2)

    def seleccionar(self):

        self.rect = self.cuadro_verdadero()

        colision = self.rect.collidelistall(self.planetas)
        if len(colision) > 0:
            colision = [self.planetas[i].get_pos() for i in colision]

            posx, posy = colision.pop(0)
            mods = pygame.key.get_mods()
            if mods & pygame.KMOD_LSHIFT:
                self.player.seleccionar_planeta_mas((posx, posy))
            else:
                self.player.seleccionar_planeta((posx, posy))

            for posx, posy in colision:
                self.player.seleccionar_planeta_mas((posx, posy))

        self.rect = None

    def cuadro_verdadero(self):
        width = self.rect.width
        height = self.rect.height
        left = self.rect.left
        top = self.rect.top
        if self.rect.left > self.rect.right:
            width = -width
            left = self.rect.right
        if self.rect.top > self.rect.bottom:
            height = -height
            top = self.rect.bottom

        return pygame.Rect((left,top), (width,height))
